﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Windows.Forms;
using HeuristicLab.Common.Resources;
using HeuristicLab.MainForm;
using HeuristicLab.MainForm.WindowsForms;
using HeuristicLab.PluginInfrastructure;

namespace HeuristicLab.Problems.Instances.Views {
  [View("ProblemInstanceConsumerView")]
  [Content(typeof(IProblemInstanceConsumer), IsDefaultView = true)]
  public sealed partial class ProblemInstanceConsumerView : AsynchronousContentView {

    public new IProblemInstanceConsumer Content {
      get { return (IProblemInstanceConsumer)base.Content; }
      set { base.Content = value; }
    }

    private IProblemInstanceProvider SelectedProvider {
      get { return (problemInstanceProviderComboBox.SelectedIndex >= 0 ? (IProblemInstanceProvider)problemInstanceProviderComboBox.SelectedItem : null); }
    }

    public ProblemInstanceConsumerView() {
      InitializeComponent();
      libraryInfoButton.Text = String.Empty;
      libraryInfoButton.Image = VSImageLibrary.Internet;
    }

    protected override void OnContentChanged() {
      base.OnContentChanged();
      if (Content == null) {
        problemInstanceProviderComboBox.DataSource = null;
      } else {
        problemInstanceProviderComboBox.DisplayMember = "Name";
        problemInstanceProviderComboBox.DataSource = GetProblemInstanceProviders().ToList();
      }
      SetEnabledStateOfControls();
    }

    public bool ContainsProviders() {
      return problemInstanceProviderComboBox.Items.Count > 0;
    }

    protected override void SetEnabledStateOfControls() {
      base.SetEnabledStateOfControls();
      problemInstanceProviderComboBox.Enabled = !ReadOnly && !Locked && Content != null && problemInstanceProviderComboBox.Items.Count > 0;
      libraryInfoButton.Enabled = SelectedProvider != null && SelectedProvider.WebLink != null;
    }

    private void problemInstanceProviderComboBox_SelectedIndexChanged(object sender, System.EventArgs e) {
      if (problemInstanceProviderComboBox.SelectedIndex >= 0) {
        problemInstanceProviderViewHost.Content = SelectedProvider;
        dynamic view = problemInstanceProviderViewHost.ActiveView;
        dynamic content = Content;
        view.Consumer = content;
        if (CheckForIProblemInstanceExporter(Content))
          view.Exporter = content;
        //SetTooltip();
      }
      SetEnabledStateOfControls();
    }

    private bool CheckForIProblemInstanceExporter(IProblemInstanceConsumer content) {
      // no check if IProblemInstanceConsumer<T> and IProblemInstanceExporter<T> have the same generic type T
      return Content.GetType().GetInterfaces()
                    .Any(x => x.IsGenericType && x.GetGenericTypeDefinition().Equals(typeof(IProblemInstanceExporter<>)));
    }

    private void comboBox_DataSourceChanged(object sender, EventArgs e) {
      var comboBox = (ComboBox)sender;
      if (comboBox.DataSource == null)
        comboBox.Items.Clear();
    }

    private void libraryInfoButton_Click(object sender, EventArgs e) {
      if (problemInstanceProviderComboBox.SelectedIndex >= 0) {
        if (SelectedProvider != null && SelectedProvider.WebLink != null)
          Process.Start(SelectedProvider.WebLink.ToString());
      }
    }

    private IEnumerable<IProblemInstanceProvider> GetProblemInstanceProviders() {
      var consumerTypes = Content.GetType().GetInterfaces()
        .Where(x => x.IsGenericType
          && x.GetGenericTypeDefinition() == typeof(IProblemInstanceConsumer<>));

      if (consumerTypes.Any()) {
        var instanceTypes = consumerTypes
          .Select(x => x.GetGenericArguments().First())
          .Select(x => typeof(IProblemInstanceProvider<>).MakeGenericType(x));

        foreach (var type in instanceTypes) {
          foreach (var provider in ApplicationManager.Manager.GetInstances(type))
            yield return (IProblemInstanceProvider)provider;
        }
      }
    }

    //protected void SetTooltip() {
    //  toolTip.SetToolTip(problemInstanceProviderComboBox, GetProviderToolTip());
    //  if (SelectedProvider.WebLink != null)
    //    toolTip.SetToolTip(libraryInfoButton, "Browse to " + SelectedProvider.WebLink.ToString());
    //  else toolTip.SetToolTip(libraryInfoButton, "Library does not have a web reference.");
    //}

    //protected string GetProviderToolTip() {
    //  var provider = SelectedProvider;
    //  string toolTip = provider.Name;

    //  if (!String.IsNullOrEmpty(provider.ReferencePublication)) {
    //    toolTip = toolTip
    //        + Environment.NewLine + Environment.NewLine
    //        + provider.ReferencePublication;
    //  }
    //  if (provider.WebLink != null) {
    //    toolTip = toolTip
    //        + Environment.NewLine
    //        + provider.WebLink.ToString();
    //  }

    //  return toolTip;
    //}
  }
}
